/************************************************************************/
/*																		*/
/*		dump_trace_file													*/
/*																		*/
/*		Richard K. Nordin			11/20/87.							*/
/*																		*/
/*		Copyright  1987-88, Apple Computer, Inc.  All rights reserved.	*/
/*																		*/
/************************************************************************/


#include <StdIO.h>
#include <Types.h>

#include "os.h"


#define	TRUE	1
#define FALSE	0


extern Boolean	Error;				/* Error Flag */
 
int				Count_a;			/* Count of -a Option Values */
int				Count_c;			/* Count of -c Option Values */
int				Count_d;			/* Count of -d Option Values */
int				Count_f;			/* Count of -f Option Values */
int				Count_i;			/* Count of -i Option Values */
int				Count_l;			/* Count of -l Option Values */
int				Count_p;			/* Count of -p Option Values */
int				Count_s;			/* Count of -s Option Values */
int				Count_t;			/* Count of -t Option Values */
	
long			Selection_a[256];	/* -a Option Values */
long			Selection_c[256];	/* -c Option Values */
long			Selection_d[256];	/* -d Option Values */
long			Selection_f[256];	/* -f Option Values */
long			Selection_i[256];	/* -i Option Values */
long			Selection_l[256];	/* -l Option Values */
long			Selection_p[256];	/* -p Option Values */
long			Selection_s[256];	/* -s Option Values */
long			Selection_t[256];	/* -t Option Values */


/*	dump_trace_file - Dump Trace File
 *
 *
 */
	
void
dump_trace_file(stream, file)
FILE	*stream;	/* Output File Stream */
FILE	*file;		/* Input File Stream */
{
	int				nitems;				/* Number of Items to Read */
	int				number;				/* Number of Items Read */
	message			msg;				/* Message */
	extern void		dump_message();		/* Dump Message */
	char			*data;				/* Associated Data */
	long			space;				/* Available Space */
	Boolean			is_selected();		/* If Value Is Selected */
	
	space = 32;
	data = (char *) malloc(space);
	if (data == 0)
	{
		fflush(stream);
		fprintf(stderr, "### DumpTrace: out of memory\n");
		Error = TRUE;
		return;
	}
	
	while (TRUE)
	{
		nitems = sizeof(message);
		number = fread(&msg, sizeof(char), nitems, file);
		if (number < nitems)
		{
			break;			/* End of File */
		}
		
		nitems = msg.mDataSize;
		if (nitems > space)
		{
			space = nitems;
			data = (char *) realloc(data, space);
			if (data == 0)
			{
				fflush(stream);
				fprintf(stderr, "### DumpTrace: out of memory, message data too large\n");
				dump_message(stderr, &msg);
				Error = TRUE;
				return;
			}
		}
		number = fread(data, sizeof(char), nitems, file);
		if (number < nitems)
		{
			break;			/* End of File */
		}

		if ((is_selected(msg.mFrom, Selection_a, Count_a) ||
			is_selected(msg.mTo, Selection_a, Count_a)) &&
			is_selected(msg.mCode, Selection_c, Count_c) &&
		    is_selected(msg.mDataPtr, Selection_d, Count_d) &&
		    is_selected(msg.mFrom, Selection_f, Count_f) &&
		    is_selected(msg.mId, Selection_i, Count_i) &&
		    is_selected(msg.mDataSize, Selection_l, Count_l) &&
		    is_selected(msg.mPriority, Selection_p, Count_p) &&
		    is_selected(msg.mStatus, Selection_s, Count_s) &&
		    is_selected(msg.mTo, Selection_t, Count_t))
		{
			dump_message(stream, &msg);
			fprintf(stream, "\n");
		
			if (msg.mDataSize != 0)
			{
				dump_memory(stream, msg.mDataPtr, data, number);
				fprintf(stream, "\n");
			}
		}
	}
	
	free(data);
	
	return;
}
