/************************************************************************************/
/*																					*/
/*				Print Manager - Display diagnostic/informative messages				*/
/*																					*/
/*				Richard W. Mincher.								5/1/87.				*/
/*				Philip Rakity									8/27/87				*/
/*				Philip Rakity - Modified for Slot 0				11/23/87			*/
/*																					*/
/*			 Copyright  1987-88, Apple Computer, Inc.  All rights reserved.		*/
/*																					*/
/************************************************************************************/

#include	<QuickDraw.h>
#include	"os.h"
#include	"managers.h"
#include	<stdio.h>
#include	<Traps.h>

#define		FALSE	0

static char			print_slot;
short				print_task = 0;
static	message		*print_message;
static	long		print_count;
static	long		print_length;
static	char		*print_buffer;
static	message		*unreg_mess;
static	void		Wait_rcv();

	pascal void illegal ()
		extern	0x4afc;
		
	pascal void SystemTask ()
		extern	_SystemTask;
	
static	char		my_object_name [] = "print manager";
static	char		my_type_name []   = "print manager";

/*
 *	print_manager - main task loop.
 */
void
main ()
{
	void			print_char();
	void			print_delay();
	tid_type		GetPrintTID ();
	void			delay ();
	void			BlockMove();
	void			unregister ();
	short			i;
	short			done;
	long			print_id;
	short			stop_print;
	message			*m;
	short			temp_task;
	char			print_buffer [256];

	InitGraf(&qd.thePort);
	OpenQueue (Wait_rcv);

	printf ("Print Manager TID = %x\n", GetTID ());
	fflush (stdout);
	
	print_message = 0;
	print_length = 0;
	print_count = 0;
	print_id = 0;
	stop_print = 0;
	print_task = 0;
	done = 0;
	
	/* extract slot for printing for start parameters	*/
	/* if none present - make it the local card			*/
	
	print_slot = GetCard ();

		
	/* if the print manager is supposed to be running on this card	*/
	/* then register the name										*/
	
	if (print_slot == GetCard ())
	{
		Net_Register_Task (my_object_name, my_type_name);

		atexit (unregister);
	}
		
		
	/* try to find the print task.  We delay a little in case the print	*/
	/* manager was delayed in starting on the other card				*/
	
	i = GetTickPS ();
	while (i && (temp_task = GetPrintTID (print_slot)) == 0)
	{
		i--;
		delay (GetTickPS () - i);
	}	

	
	print_task = temp_task;
	
	if (print_slot != GetCard ())
	{
		exit (0);
		illegal ();
	}
	
	/*	Event Loop	*/

	print_buffer [255] = '\0';
	
	printf ("Starting Main Loop\n");
	fflush (stdout);
	while(!done)
	{
		m = Receive( OS_MATCH_ALL, OS_MATCH_ALL, OS_MATCH_ALL, OS_NO_TIMEOUT, 0 );	/*	Receive any message, no timeout		*/
		if ((long) m > 0)
		{
			switch(m->mCode)
			{
				case PRINT_ME:
					if(!stop_print)
					{
						print_count = 0;
						print_length = m->mDataSize;
						
						if (print_length > 255)
							print_length = 255;		
						
						if ((m->mStatus = NetCopy ( m->mFrom, m->mDataPtr, 
							GetTID(), print_buffer, print_length)) == OS_NO_ERRORS)
						{
						
							print_buffer [print_length] = '\0';
							
							i = print_length;
							while (i-- > 0)
							{
								if (print_buffer [i] == '\012')
									print_buffer [i] = ' ';
							}
							fputs (print_buffer, stdout);
							fflush (stdout);
						}
					}
					
					m->mTo = m->mFrom;
					m->mFrom = print_task;
					m->mCode = m->mCode + 1;
					Send( m );
	
					break;
				case PRINT_ME+1:
					illegal ();
					break;
				case PRINT_READY:
					illegal ();
					break;
				case PRINT_START:
					stop_print = 0;
					break;
				case PRINT_STOP:
					stop_print = 1;
					break;
				case MC_STOP:
					illegal ();
					break;
				default:
					FreeMsg( m );
			}
		}
	}
	
}



static tid_type GetPrintTID (slot)
char	slot;
{
	tid_type			printTID;
	struct ra_GetCards	get_cards;
	message				*msgptr;
	unsigned short		index;

	printTID = 0;
	
	if (GetICCTID () != 0)
	{
		if ((msgptr = GetMsg ()) == NULL)
			return (printTID);
	
		msgptr -> mCode = ICC_GETCARDS;
		msgptr -> mDataPtr = (char *) &get_cards;
		msgptr -> mDataSize = sizeof (struct ra_GetCards);
		msgptr -> mTo = GetICCTID ();
		Send (msgptr);
		
		msgptr = Receive (OS_MATCH_ALL, OS_MATCH_ALL, ICC_GETCARDS+1,  OS_NO_TIMEOUT, 0);
		
		if ((long) msgptr > 0)
		{
			if (msgptr -> mStatus == 0)
			{
				if (get_cards.tid [slot] > 0)
				{
					index = 0;
					printTID = Lookup_Task (my_object_name, my_type_name, get_cards.tid [slot], &index);
				}
			}
			FreeMsg (msgptr);
		}
		else
			return (printTID);
	}
	else
	{
		index = 0;
		printTID = Lookup_Task (my_object_name, my_type_name, GetNameTID (), &index);
	}		

	return (printTID);
}

static void delay (timeout)
long timeout;
{
	/* Delay is done by waiting for a message that will never arrive! */
	
	(void) Receive (OS_MATCH_NONE, OS_MATCH_NONE, OS_MATCH_NONE, timeout, 0);	
}

void unregister ()
{
			
	CloseQueue ();
	printf ("CloseQueue Called\n");
	fflush (stdout);
}

void Wait_rcv()
{
	SystemTask();
}